function plot_from_dump(fn; iter=1, mode=1, collimate=false)
    z0, meta = h5open(fn, "r") do fid
        rx = fid["RXFields"]
        _colons = ntuple(_ -> Colon(), ndims(rx) - 1)

        getindex(rx, _colons..., iter), Dict(attrs(fid))
    end

    # third dimension can contain fields for different TX modes
    z = z0[:, :, mode]

    Dx = meta["Dx"]
    Dy = meta["Dy"]
    Nx = meta["Nx"]
    Ny = meta["Ny"]

    x, y = fftfreq(Nx, Dx[end]), fftfreq(Ny, Dy[end])

    if collimate
        λ = meta["lambda"]
        Dz = meta["Dz"]
        yt = transpose(y)
        r² = @. x^2 + yt^2
        @. z *= cis(π / (λ * Dz) * r²)
    end

    tr = complex_plot(; x, y, z=fftshift(z))
    lt = Config(; xaxis=Config(title="x"), yaxis=Config(title="y", scaleanchor="x", autorange=true))
    Plot(tr, lt)
end

function attributes_from_dump(fn; disp=false)
    meta = h5open(fn, "r") do fid
        OrderedDict(attrs(fid))
    end
    disp && show(IOContext(stdout, :limit => false), "text/plain", meta)

    return meta
end

function post_dump_sampling_check(fn; kwargs...)

    meta = attributes_from_dump(fn)
    N = meta["Nx"]
    dx1, dxn = extrema(meta["dx"])
    D1, Dn = extrema(meta["Dx"])
    λ = meta["lambda"]
    r0 = meta["r0eff"]

    z = range(0, meta["Dz"], meta["Nz"])

    check_sampling(z, N, dx1, D1, dxn, Dn; λ, r0, kwargs...)
end