include(joinpath("..", "src", "sampling_requirements.jl"))
include(joinpath("..", "src", "plot_basics.jl"))

function plot_geom_sampling_reqs(z, D1, Dn=D1; λ=1.55e-6, Nd=251, bw=1 / 1e-2, relax=false)

    # at least 10 samples within the ROIs D1 and Dn
    dx1 = range(D1 / 100, D1 / 10, Nd)
    dxn = range(Dn / 100, Dn / 10, Nd) 
    dxnt = transpose(dxn)

    # compare my notes on the relaxation of the sampling requirement (2) for a source with bandwidth < 1/dx1, e.g., bw ~ 1/w0 = 1 / 1e-2
    c = if relax
        @. inv(bw * dx1)
    else
        1
    end

    # second requirement
    Nmin = @. D1 / 2dx1 + Dn / dxnt + λ * z / (2 * dx1 * dxnt * c)
    # first requirement
    dxnmax = @. (λ * z - Dn * dx1) / D1

    # 2D representation of second requirement
    z = log2.(Nmin)
    # enforce first requirement
    @. z[dxnt > dxnmax] = Inf

    plt = plot.contour(x=dx1 * 1e3, y=dxn * 1e3, z=z, contours=Config(showlabels=true, labelfont=Config(color="white"), size=2), colorbar=Config(title=Config(text="log<sub>2</sub>(N<sub>min</sub>)"))) # predefined levels not working - for whatever reason -.-

    plt.layout.xaxis.title = "dx1 / mm"
    plt.layout.yaxis.title = "dxn / mm"
    # plt.layout.yaxis.scaleanchor = "x"
    plt.layout.yaxis.autorange = true
    plt.layout.title = "Sampling Requirements 1 & 2"

    display(plt)
    return
end